from Products.CMFDefault.Document import Document

ADD_CONTENT_PERMISSION = 'Add portal content'

import Globals, StructuredText, string, Products.CMFDefault.utils
from StructuredText.HTMLWithImages import HTMLWithImages
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.utils import _format_stx, keywordsplitter
from Products.CMFDefault.utils import parseHeadersBody, SimpleHTMLParser, bodyfinder, _dtmldir
from OFS.Image import Image

factory_type_information = ( { 'id'             : 'ImageDoc'
                             , 'meta_type'      : 'ImageDoc'
                             , 'description'    : """\
ImageDocs can contain both text and images."""
                             , 'icon'           : 'ImageDoc_icon.gif'
                             , 'product'        : 'CMFImageDoc'
                             , 'factory'        : 'addImageDoc'
                             , 'immediate_view' : 'metadata_edit_form'
                             , 'actions'        :
                                ( { 'name'          : 'View'
                                  , 'action'        : 'ImageDoc_view'
                                  , 'permissions'   : (
                                      CMFCorePermissions.View, )
                                  }
                                , { 'name'          : 'Edit'
                                  , 'action'        : 'ImageDoc_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                , { 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                )
                             }
                           ,
                           )

def addImageDoc(self, id, title='', description='', text_format='',
                text='', pic='', picpos=''):
    """ Add an ImageDoc """
    o = ImageDoc(id, title, description, text_format, text, pic, picpos)
    self._setObject(id,o)


class CMFHtmlWithImages(HTMLWithImages):
    """ Special subclass of HTMLWithImages, overriding document() """
    def document(self, doc, level, output):
        """\
        HTMLWithImages.document renders full HTML (head, title, body).  For
        CMF Purposes, we don't want that.  We just want those nice juicy
        body parts perfectly rendered.
        """
        for c in doc.getChildNodes():
           getattr(self, self.element_types[c.getNodeName()])(c, level, output)

CMFHtmlWithImages = CMFHtmlWithImages()



class ImageDoc(Document):


  meta_type="ImageDoc"
  

  
  def __init__(self, id, title='', description='', text_format='', text='', 
               pic=None, picpos='top-left'):
      """Class constructor"""
      Document.__init__(self, id, title, description, text_format, text)
      self.pic=pic
      self.picpos = picpos
      
  def _edit(self, text_format, text, file='', safety_belt='', pic=None, picpos=None, removepic=None):
    self.picpos = picpos

    if removepic:
        self.pic = None

    if pic and (type(pic) is not type('')) and string.strip(pic.filename) != '':
      tempImage = Image("ImageDoc_pic", "ImageDoc pic", pic)
      if tempImage:
        self.pic = tempImage

    Document._edit(self, text_format, text, file, safety_belt)
  
  #this workflowaction is at class scope
  edit = WorkflowAction(_edit)
      
  def manage_editImageDoc(self, text_format, text, file='', REQUEST=None, pic=None, picpos=None, removepic=None):
        """ A ZMI (Zope Management Interface) level editing method """

        self._edit(text_format, text, file, pic=pic, picpos=picpos, removepic=removepic)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url()
                + '/manage_edit'
                + '?manage_tabs_message=ImageDoc+updated&pic=%s' % (pic,)
                )


  def get_pic(self, REQUEST, RESPONSE):
    """ Return the pic """
    if not self.pic:
        return None
    #REQUEST.RESPONSE.setHeader('Content-Type', 'application/octet-stream')
    #Patched to work with opera, submited by Sigve Tjora
    REQUEST.RESPONSE.setHeader('Content-Type',self.pic.getContentType())
    return self.pic.data


  def CookedBody(self, stx_level=None, setlevel=0):
      """Return an HTML string containing the data in our ImageDoc"""

      # Use the Document class's CookedBody method, then grab the results and
      # add in our image where we want it"""
      CookedHTML = Document.CookedBody(self, stx_level, setlevel)
      if not (self.picpos and self.pic):
          return CookedHTML

      # possible imagepositions are:
      #   vertical:   top, bottom, centre
      #   horizontal: left, right, centre
      # all combinations are valid except centre-centre

      vpos, hpos = string.split(self.picpos, '-')
      absolute_url = self.absolute_url()
      # 1. align horizontally
      if hpos == 'centre':
          imgtag = '<center><img align="center" src="%s/get_pic"></center>'%absolute_url
      else:
          imgtag = '<img align="%s" src="%s/get_pic">' % (hpos,absolute_url)
      
      # 2. align vertically    
      if vpos == "centre":
          middle = int(len(CookedHTML) / 2)
          temp = middle
          #Work out if were in a tag, if so move to end of tag
          nextclose=string.find(CookedHTML,">",middle)
          nextopen=string.find(CookedHTML,"<",middle)
          if nextclose < nextopen:
              middle=nextclose+1
          return CookedHTML[:middle] + imgtag + CookedHTML[middle:]
      elif vpos == "bottom":
          return CookedHTML + imgtag
      else: #vpos must be top
          return imgtag + CookedHTML
      
      
InitializeClass(ImageDoc)
